/****************************************************************************
 *   $Id:: main.c 5034 2010-09-24 23:48:37Z nxp12832                        $
 *   Project: NXP LPC11xx CAN example
 *
 *   Description:
 *     This file is part of the CAN and CANopen on-chip driver examples.
 *
 ****************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
****************************************************************************/

#include <cr_section_macros.h>
#include "driver_config.h"
#include "target_config.h"
#include "small_gpio.h"
#include "gpio.h"
#include "rom_drivers.h"

#ifndef NULL
#define NULL    ((void *)0)
#endif

__BSS(RESERVED) char CAN_driver_memory[256]; // reserva 256 bytes for CAN driver

ROM **rom = (ROM **)0x1fff1ff8;

CAN_MSG_OBJ msg_obj;
uint32_t cuenta;
uint32_t cuenta_Rx;
uint32_t cuenta_Tx;

/* Initialize CAN Controller */
uint32_t ClkInitTable[2] = {
  0x00000000UL, // CANCLKDIV
  0x00001C57UL  // CAN_BTR
};

/* Callback function prototypes */
void CAN_rx(uint8_t msg_obj_num);
void CAN_tx(uint8_t msg_obj_num);
void CAN_error(uint32_t error_info);

/* Publish CAN Callback Functions */
CAN_CALLBACKS callbacks = {
   CAN_rx,
   CAN_tx,
   CAN_error,
   NULL,
   NULL,
   NULL,
   NULL,
   NULL,
};

/*	CAN receive callback */
/*	Function is executed by the Callback handler after
	a CAN message has been received */
void CAN_rx(uint8_t msg_obj_num){

	//ToggleGPIOBit(LED_PORT, LEDV_BIT);
	cuenta_Rx++;

  return;
}

/*	CAN transmit callback */
/*	Function is executed by the Callback handler after
	a CAN message has been transmitted */
void CAN_tx(uint8_t msg_obj_num){

	ToggleGPIOBit(LED_PORT, LEDV_BIT);
	cuenta_Tx++;
	return;
}

/*	CAN error callback */
/*	Function is executed by the Callback handler after
	an error has occured on the CAN bus */
void CAN_error(uint32_t error_info){
  return;
}

/*	CAN interrupt handler */
/*	The CAN interrupt handler must be provided by the user application.
	It's function is to call the isr() API located in the ROM */
void CAN_IRQHandler (void){
  (*rom)->pCAND->isr();
}

int main(void) {

	InitGPIO();

	/* Initialize the CAN controller */
	(*rom)->pCAND->init_can(&ClkInitTable[0], 1);

	/* Configure the CAN callback functions */
	(*rom)->pCAND->config_calb(&callbacks);


	SetGPIOOut(LED_PORT, LEDV_BIT);
	SetGPIOOut(LED_PORT, LEDR_BIT);
	SetGPIOOut(PORT_0, PIO0_3);
	GPIOSetValue( PORT_0, PIO0_3, 0 ); // Habilitación del transceiver CAN vía STB (en bajo --> Activo)


    //set loopback mode
    LPC_CAN->CNTL |= (1<<7);
    LPC_CAN->TEST |= (1<<4);

	/* Enable the CAN Interrupt */
	NVIC_EnableIRQ(CAN_IRQn);


	/* Configure message object 1 to receive all 11-bit messages 0x200-0x4FF */
	msg_obj.msgobj = 0;
	msg_obj.mode_id = 0x7FF; //| CAN_MSGOBJ_STD;
	msg_obj.mask = 0x000;
	(*rom)->pCAND->config_rxmsgobj(&msg_obj);




	while(1){

		if (cuenta == 1000000){

			ToggleGPIOBit(LED_PORT, LEDR_BIT);
			cuenta = 0;

			msg_obj.msgobj  = 1;
			msg_obj.mode_id = 0x7FF;
			msg_obj.mask    = 0x0;
			msg_obj.dlc     = 4;
			msg_obj.data[0] = 'T';	//0x54
			msg_obj.data[1] = 'E';	//0x45
			msg_obj.data[2] = 'S';	//0x53
			msg_obj.data[3] = 'T';	//0x54
			(*rom)->pCAND->can_transmit(&msg_obj);


		}
		cuenta++;
	}
}
